/********************************************************************************
 [Jedi]'s Link Cleaner v1.02
 Configure the following variables and compile with: gcc -o oldclean oldclean.c

****** Configure me *************************************************************/
#define VERSION 1.02


/* TESTING_MODE - if enabled lnclean will not delete the bad links
*/

//#define TESTING_MODE

/***********************************************************************************/


#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <time.h>

void recurse_path(const char *path);
void checklink(char *path);

time_t now;
int recurse = 0 , verbose = 0;
unsigned long goodlinks = 0, badlinks = 0;

void printhelp(int argc, char **argv) {
	printf("Usage: %s [-n|-r path] [-c chrootpath] [-v]\n", argv[0]);
	printf(" -r    - recurse path (recommended)\n");
	printf(" -n    - do not recurse path\n");
	printf(" -c    - chroot to chrootpath, then run the test on path. (use with crontab see readme)\n");
	printf(" -v    - be verbose\n");
	printf(" -h/?  - this help screen.\n");
	printf("if no switches provided, default check would be non-recursive for current dir. ");
	printf("(same as running \"%s -n .\")\n", argv[0]);
}

int main(int argc, char **argv) {
	char buff[MAXPATHLEN] = "./";
	struct stat bufstat;
	int c;

	printf("-- [Jedi]'s Link Cleaner v%.2f --\n", VERSION);
	while((c = getopt(argc, argv, "r:n:c:?hv")) != -1) {
	   switch (c) {
		case 'h':
		case '?':
		case ':':
			printhelp(argc, argv);
			exit(0);
			break;
		case 'r':
		case 'n':
			recurse = (c == 'r') ? 1 : 0;
			if (strlen(optarg) > MAXPATHLEN) {
				fprintf(stderr, "ERROR: Path length is too long!\n");
				exit(1);
			}
			strncpy(buff, optarg, MAXPATHLEN-1);
			break;
		case 'c':
			if (chroot(optarg)) {
				fprintf(stderr, "Error %d: %s\n",errno, strerror(errno));
				exit(1);
			}
			break;
		case 'v':
			verbose = 1;
			break;
	   }
	}							
	if ( stat(buff, &bufstat) ) {
		fprintf(stderr, "Error %d: %s\n",errno, strerror(errno));
		exit(1);
	} else if (!S_ISDIR(bufstat.st_mode)) {
		fprintf(stderr, "Path must be a directory !\n");
		exit(1);
	}

	now = time(NULL);
	if (verbose) printf("Checking (%s) Recuse mode: (%s) Testing mode: (%s)\n", buff, recurse ? "On" : "Off", 
		#ifdef TESTING_MODE
			"On"
		#else
			"Off"
		#endif
		 	);

	recurse_path(buff);
	if (verbose) printf("------------------------------------\n");
	printf("Bad links: %ld , Good links: %ld\n", badlinks, goodlinks);
	printf("Link cleaning took %ld seconds.\n", time(NULL)-now);
	return 0;
}

void checklink(char *path) {
	struct stat bufstat, bufstat2;

	if (verbose) printf("Checking link %s - ",path);
	if( lstat(path, &bufstat) == 0 && stat(path, &bufstat2) == -1) {
		if (verbose) printf("Bad link!\n");
		badlinks++;
		#ifndef TESTING_MODE
		unlink(path);
		#endif
	} else {
		goodlinks++;
		if (verbose) printf("OK\n");
	}
}

void recurse_path(const char *path) {
	DIR *curSecDir;
	struct dirent *currentdir;
	struct stat dirstat;
	char tpath[MAXPATHLEN];

	curSecDir = opendir(path);
	if (curSecDir == NULL) {
		fprintf(stderr, "Could not open dir %s (%s)\n",path, strerror(errno));
		return;
	}
//	printf("Recusing %s\n", path);
	while ((currentdir = readdir(curSecDir)) != NULL) {
		if (strcmp(currentdir->d_name, ".")==0 || strcmp(currentdir->d_name,"..")==0) continue;
		sprintf(tpath,"%s/%s",path,currentdir->d_name);
		lstat(tpath, &dirstat);
		if (S_ISDIR(dirstat.st_mode) && recurse) /* Recurse path */
			recurse_path(tpath);
		else if (S_ISLNK(dirstat.st_mode)) {
			checklink(tpath);
		}
	}
	closedir(curSecDir);
//	printf("Decursing %s!!!\n", path);
}

